﻿//-----------------------------------------------------------------------
// <copyright file="DemoGuiModule.cs" company="dataPartner">
//     Copyright (c) dataPartner s. r. o.  All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BioWes.Sdk.DemoModule
{
    using System;
    using System.Diagnostics.Contracts;
    using System.Windows.Forms;
    using BioWes.ProjectManager;
    using BioWes.Sdk.DemoModule.Commands;
    using dataPartner.ApplicationFramework4;
    using dataPartner.ApplicationFramework4.Commands;
    using dataPartner.ApplicationFramework4.Face;
    using dataPartner.ComponentModel;


    /// <summary>DemoGuiModule</summary>
    public class DemoGuiModule : BaseFaceModule,
        Services.IDemoModuleService
    {
        #region Properties

        private Ribbon _moduleRibbon;

        private RibbonTabCategory _demoModuleTabCategory;

        private Face.DemoModuleForm _demoModuleForm;

        #endregion

        #region Constructor

        /// <summary>Constructor</summary>
        public DemoGuiModule()
        {
        }

        #endregion

        #region Override methods

        /// <summary>Run module</summary>
        public override void Run()
        {
            base.Run();

            ((IServiceRegistry)AppFramework).RegisterService(typeof(Services.IDemoModuleService), this);

            CreateRibbon();
            AppFramework.Face.Ribbon.Merge(_moduleRibbon);
        }

        /// <summary>Stop module</summary>
        public override void Stop()
        {
            ((IServiceRegistry)AppFramework).UnregisterService(typeof(Services.IDemoModuleService));

            base.Stop();
        }

        /// <summary>Configure module</summary>
        public override void Configure()
        {
            base.Configure();
        }

        /// <summary>Module is activated</summary>
        protected override void OnActivated()
        {
            // Při aktivaci modulu přepni na hlavní okno modulu

            base.OnActivated();
        }

        /// <summary>Module is deactivated</summary>
        protected override void OnDeactivated()
        {
            AppFramework.Face.Ribbon.Unmerge(_moduleRibbon);

            // Zavři aktivní formulář modulu při deaktivaci modulu

            base.OnDeactivated();
        }

        #endregion

        #region Methods

        private void CreateRibbon()
        {
            Contract.Assert(_moduleRibbon == null);

            _moduleRibbon = AppFramework.Face.Ribbon.CreateRibbon();

            // register ShowDemoModuleCommand to ProtocolManager
            var protocolsTab = _moduleRibbon.Tabs.CreateTab(ProjectManager.Commands.ProjectManagerUINames.ProtocolsTab);
            var category = _moduleRibbon.TabCategories.CreateTabCategory(ProjectManager.Commands.ProjectManagerUINames.ProtocolsTabCategory);
            var demoModuleExternalGroup = protocolsTab.Groups.CreateGroup(DemoModuleUIName.DemoModuleExternalRibbonGroup, 10); // 10 is order of group in another ribbon

            // if you can't implemented Executed handler inside command class (then DemoModuleCommand.HasDefaultHandler = false)
            ICommand cmd;
            cmd = new ShowDemoModuleCommand();
            AppFramework.CommandManager.Commands.Add(cmd);
            cmd.Executed += ShowDemoModule_Executed;
            demoModuleExternalGroup.Commands.Add(cmd);

            var demoModuleFirstTab = _moduleRibbon.Tabs.CreateTab(DemoModuleUIName.DemoModuleFirstRibbonTab);
            var demoModuleGroup = demoModuleFirstTab.Groups.CreateGroup(DemoModuleUIName.DemoModuleRibbonGroup);

            cmd = new DemoModuleCommand();
            AppFramework.CommandManager.Commands.Add(cmd);
            demoModuleGroup.Commands.Add(cmd); // can be repeated with others commands and other groups

            _demoModuleTabCategory = _moduleRibbon.TabCategories.CreateTabCategory(DemoModuleUIName.DemoModuleTabCategory);
            _demoModuleTabCategory.Visible = false;
            demoModuleFirstTab.Category = _demoModuleTabCategory;
        }

        #endregion

        #region Events

        private void ShowDemoModule_Executed(object sender, System.ComponentModel.HandledEventArgs e)
        {
            e.Handled = true;

            var protocolManager = AppFramework.GetService<IProtocolManagerService>();
            if (protocolManager != null && protocolManager.SelectedProtocol != null)
            {
                // for example you can: 
                // --------------------

                // 1) show your form
                ShowDemoModuleForm(true);

                // or

                // 2) call DemoModuleComand
                //var command = AppFramework.CommandManager.Commands[CommandName.DemoModuleCommand];
                //command.Execute();

                // 3) another work....
            }
        }

        #endregion

        #region DemoModuleForm Members

        private void ShowDemoModuleForm(bool activate)
        {
            if (_demoModuleForm == null)
            {
                _demoModuleForm = new Face.DemoModuleForm(AppFramework);
                _demoModuleForm.Activated += DemoModuleForm_Activated;
                _demoModuleForm.Deactivate += DemoModuleForm_Deactivate;
                _demoModuleForm.FormClosed += _demoModuleForm_FormClosed;

                //((IServiceRegistry)AppFramework).RegisterService(typeof(IDemoModuleFormProvider), _demoModuleForm); // for example, if you need it
            }

            if (AppFramework.Face.Windows.ContainsKey(Face.FormName.DemoModuleForm))
            {
                IWindow window = AppFramework.Face.Windows[Face.FormName.DemoModuleForm];
                if (window != _demoModuleForm)
                {
                    // nemelo by nastat
                    AppFramework.Face.Windows.Remove(Face.FormName.DemoModuleForm);
                    AppFramework.Face.Windows[Face.FormName.DemoModuleForm] = _demoModuleForm;
                }
            }
            else
            {
                AppFramework.Face.Windows[Face.FormName.DemoModuleForm] = _demoModuleForm;
            }
        }

        private void DemoModuleForm_Activated(object sender, EventArgs e)
        {
            _demoModuleTabCategory.Show(RibbonTabCategoryShowOption.ActivatePreviouslySelectedTabOrFirst); // activate DemoModule ribbon category

            // next code after DemoModuleForm is activated
        }

        private void DemoModuleForm_Deactivate(object sender, EventArgs e)
        {
            // something code after DemoModuleForm is deactivated
        }

        private void _demoModuleForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // ((IServiceRegistry)AppFramework).UnregisterService(typeof(IDemoModuleFormProvider)); // for example, if you need it

            if (AppFramework.Face.Windows.ContainsKey(Face.FormName.DemoModuleForm))
            {
                AppFramework.Face.Windows.Remove(Face.FormName.DemoModuleForm);
            }

            _demoModuleForm = null;

            // next code after DemoModuleForm is closed
        }

        #endregion

        #region IDemoModuleService Members

        void Services.IDemoModuleService.RecalculateData()
        {
            AppFramework.Face.ShowMessageBox("Now you can process any data from protocol.", null, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

            // for example: your code that working with your data...
            // ...
        }

        #endregion
    }
}
